#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <glib.h>
#include "getcpu.h"

/* All of this file is basically cutted and pasted from libgtop2. -ssp
 */

#define FILENAME        "/proc/stat"

static void
file_to_buffer (char *buffer, const char *filename)
{
    char *content;
    gsize len;
    
    if (!g_file_get_contents (filename, &content, &len, NULL))
	return;
    
    len = MIN (len, BUFSIZ-1);
    
    memcpy (buffer, content, len);
    
    g_free (content);
}

static inline char *
skip_line (const char *p)
{
    while (*p && *p != '\n') p++;
    return (char *) (*p ? p+1 : p);
}

static inline char*
next_token(const char *p)
{
    while (isspace(*p)) p++;
    return (char*) p;
}

static inline char *
skip_token (const char *p)
{
    p = next_token(p);
    while (*p && !isspace(*p)) p++;
    p = next_token(p);
    return (char *)p;
}

static int
read_n_cpu (void)
{
    char buffer [BUFSIZ], *p = buffer;
    int ncpu;
    
    file_to_buffer(buffer, FILENAME);
    
    for (ncpu = 0; ncpu < NCPU_RHEL4; ncpu++) {
	
	p = skip_line(p);
	
	if (strncmp (p, "cpu", 3) || !isdigit (p [3]))
	    break;

	p = skip_token(p);
    }
    
#if DEBUG
    printf ("\nThis machine has %d CPUs.\n\n", ncpu);
#endif

    return ncpu;
}


void
get_cpu_rhel4 (cpu_rhel4 *buf)
{
    char buffer [BUFSIZ], *p;
    int i;
    int ncpu = read_n_cpu();
    
    memset (buf, 0, sizeof (cpu_rhel4));
    
    file_to_buffer(buffer, FILENAME);
    
    /*
     * GLOBAL
     */
    
    p = skip_token (buffer);        /* "cpu" */
    
    buf->user = strtoull (p, &p, 0);
    buf->nice = strtoull (p, &p, 0);
    buf->sys  = strtoull (p, &p, 0);
    buf->idle = strtoull (p, &p, 0);
    
    /* 2.6 kernel */
    buf->idle += strtoull(p, &p, 0); /* "iowait" */
    buf->sys  += strtoull(p, &p, 0); /* "irq" */
    buf->sys  += strtoull(p, &p, 0); /* "softirq" */
    
    buf->total = buf->user + buf->nice + buf->sys + buf->idle;
    
    buf->frequency = 100;
    
    /*
     * PER CPU
     */
    
    for (i = 0; i < NCPU_RHEL4 && i < ncpu; i++) {
	
	p = skip_line(p); /* move to ^ */
	
	if (strncmp (p, "cpu", 3) || !isdigit (p [3]))
	    break;
	
	p = skip_token(p); /* "cpuN" */
	
	buf->xcpu_user [i] = strtoull (p, &p, 0);
	buf->xcpu_nice [i] = strtoull (p, &p, 0);
	buf->xcpu_sys  [i] = strtoull (p, &p, 0);
	buf->xcpu_idle [i] = strtoull (p, &p, 0);
	
	/* 2.6 kernel */
	buf->xcpu_idle [i]  += strtoull(p, &p, 0); /* "iowait" */
	buf->xcpu_sys  [i]  += strtoull(p, &p, 0); /* "irq" */
	buf->xcpu_sys  [i]  += strtoull(p, &p, 0); /* "softirq" */
	
	buf->xcpu_total[i] = buf->xcpu_user [i]				\
	    + buf->xcpu_nice [i]					\
	    + buf->xcpu_sys  [i]					\
	    + buf->xcpu_idle [i];
    }
}



